/*
 * RolloutMonteCarlo.h
 *
 *  Created on: Apr 15, 2010
 *      Author: dbjorge
 *
 * Copyright 2010 Worcester Polytechnic Institute
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef ROLLOUTMONTECARLO_H_
#define ROLLOUTMONTECARLO_H_

#include "../search/SearchAlgorithm.h"
#include "../gametree/Action.h"
#include "../gametree/RandomGenerator.h"
#include <float.h>

/**
 *  This simple random rollout-based Monte-Carlo search forms the
 *  basis of most of the other Monte-Carlo algorithms to be tested.
 *
 *  In addition to dealing with some of the basic work similar to all
 *  Monte-Carlo searches, it provides a simulate() function which should
 *  prove to be a significant time savings in execution speed of most
 *  algoirthms. simulate() allows the simulation of a random monte-carlo
 *  playout which uses the minimax tree structure to generate a simulated
 *  winner without the overhead associated with actually evaluating a full
 *  chain of nodes of size tree.maxDepth.
 *
 *  In general, overriding classes should be able to get away with just modifying
 *  the arm selection policy determined by the select() method and possibly
 *  the consideration termination policy specified by the shouldPlayout()
 *  method. To enable this functionality and not have to bother overriding
 *  more primitive methods, classes need only ensure that the estimateReward()
 *  function return a valid empirical measure of a node's value (NOT necessarily
 *  the value which select() maximizes). The difference is that select() may wish
 *  to incorporate random choice to ensure exploration - in general, estimateReward()
 *  should be a representation of actual value, for use with currentBestMove().
 *
 *  In fact, the default behavior of estimateReward() is simply to return the
 *  searchData.empiricalValue property of the state in question. This is probably
 *  sufficient for most algorithms.
 *
 *  The basic layout of protected methods is given below - in general, most algorithms
 *  will need only to modify the evaluate() method, so long as they use empiricalValue
 *  as intended (as an estimation of minimax value). Some may also wish to modify the
 *  shouldPlayout() method, but the default behavior will be suitable for most algorithms.
 *
 *  iterate()
 *      search()
 *          shouldPlayout()
 *          simulate()
 *          select()
 *              evaluate()
 *              evaluateFirst()
 *          estimateValue()
 *          updateValue()
 *
 *  currentBestMove()
 *      estimateValue()
 */
class RolloutMonteCarlo : public SearchAlgorithm {
public:
	/**
	 * Constructs a new Rollout-based Monte-Carlo search algorithm
	 * based on the given parameters.
	 *
	 * @param simulationEffectiveness A measure of the effectiveness of any heuristics
	 * to be emulated by simulate()'s random playout policy. This should be
	 * a value in the range [0,1], where 0 represents no heuristic usage at
	 * all (a simulate()ed playout is purely random) and 1 represents a perfect
	 * heuristic (a simulate()ed playout always assumes both players play optimally)
	 *
	 * @param propogationConstant A measure of how deeply final simulated playouts
	 * should be propogated. This should be a value in the range [0,1]. 0 represents
	 * no propogation whatsoever, that is, if a search() calls another search() recursively,
	 * the inner search()'s value is not used in the outer search()'s return value
	 * (instead favoring the internal representation from estimateValue()). 1 represents
	 * propogating the inner search()'s value exactly, completely ignoring the estimated
	 * reward of the state in question.
	 */
	RolloutMonteCarlo(double simulationEffectiveness, double propogationConstant);
	virtual ~RolloutMonteCarlo();

	/*
	 * See SearchAlgorithm::iterate().
	 *
	 * Default implementation simply runs a search() from the root to update tree state.
	 */
	virtual void iterate();

	/*
	 * See SearchAlgorithm::currentBestMove().
	 *
	 * Default implementation simply returns the action from the root which maximizes
	 * the value returned by estimateValue()
	 */
	virtual action_t currentBestMove();

	/**
	 * See SearchAlgorithm::toString()
	 *
	 * Don't forget to override this!
	 */
	virtual std::string toString() const =0;

private:
	/**
	 * The perceived effectiveness of the simulation phase of the algorithm. Ranges from 0
	 * to 1, where 0 indicates the simulation has no ability to discern from random play and
	 * 1 indicates perfect accuracy in determining the minimax winner.
	 */
	double mEffectiveness;

	/**
	 * Value in [0,1] which determines whether search playout value should be propogated
	 * as-is (1) or whether it should be ignored in favor of just propogating based on the
	 * estimateValue() result of the next node in the search path (0), or something in between
	 */
	double mPropogationConstant;

protected:
	/**
	 * This random number generator is initialized by the base class, but
	 * is made protected for the convenience of its subclasses. Redefining
	 * or reseeding the generator will result in undefined behavior of
	 * base class methods utilizing it (notably, simulate()), so don't do that.
	 */
	RandomGenerator mGenerator;

	/**
	 * This method should attempt to run a directed monte-carlo style playthrough
	 * of the (sub)tree rooted at the given starting state. No guarantees are
	 * made about the state - it may be terminal, or it may be the root node.
	 *
	 * The method should determine a value to the playthrough based on the results
	 * of the playout. This value should be representative of an estimate of minimax
	 * value, that is, it should be higher to represent a favorable outcome for the
	 * maximizing player (B) and lower to represent a favorable outcome for the minimizing
	 * player (A). For consistency it is recommended that this score remain bounded
	 * within a fixed interval. In the default implementation, this interval is [0,1].
	 *
	 * In addition to returning this value as the results of the search, this method
	 * should update the searchData structure of the relevant bits of the tree with
	 * the results of the playout for future consideration.
	 *
	 * The default implementation uses the termination policy defined by shouldPlayout()
	 * to determine whether it should run a pure monte-carlo playout with simulate() or
	 * recursively search() a select()ed subnode. In either case, the resulting playout
	 * data is added to the internal state of the startingState with the update() method
	 * prior to this method returning.
	 *
	 * @param startState The state from which to begin the search.
	 * @return The score which the search playout resulted in, which should
	 * be a measure representing how well the search went - higher is better
	 * for player B, lower is better for player A.
	 */
	virtual double search(GameState& startState);

	/**
	 * This method should determine the search policy of the specific
	 * algorithm. It should return the next action which should be played
	 * from a given starting state, which may be any non-terminal node.
	 * For example, in UCT, this function would return the action which
	 * results in a child state of the maximum UCT score of the parent
	 * state's children.
	 *
	 * The default implementation of this method runs the evaluate() method
	 * over every child which has already been visited, and runs the evaluateFirst()
	 * method as well. If the maximum result of the evaluate() calls is greater than
	 * the result of the evaluateFirst() call, this implementation returns the
	 * child which generated the maximum evaluate() value. If the evaluateFirst() call
	 * returns a greater result than all of the evaluate() calls, this method returns
	 * a random action which results in a previously unvisited child node.
	 *
	 * In short, subclasses will generally be better off simply redefining the
	 * evaluate() and evaluateFirst() methods, though if special processing is
	 * needed this method is able to be overriden directly.
	 *
	 * @param startState The starting state from which we'd like to
	 * select a child to continue the playout with. Must be non-terminal.
	 * @return The action index of the child which has been selected
	 * for playout.
	 */
	virtual action_t select(GameState& startState);

	/**
	 * This method defines the playout termination policy of a search. That is,
	 * this method determines which a search should continue recursively or when it
	 * should simply begin to playout pseudorandomly according to the simulate()
	 * method. This method does NOT need to concern itself with checking for terminal
	 * nodes, which is the responsibility of the search() method - the input state may
	 * be assumed to be non-terminal.
	 *
	 * The default implementation of this method says to perform a random playout for nodes
	 * which have not yet been visited and to perform a recursive search whenever a node has
	 * been previously visited. Though suitable for most environments, some methods may wish
	 * to modify this behavior. In particular, it is possible to restrict difficult searches
	 * to a limited number of plies by having this method cut off at a certain state depth.
	 *
	 * @param state The state to consider for continued playout.
	 * @return Whether the remaining playout should be based on a simple semi-random
	 * simulate() call (true) or should recursively perform the more complex search()
	 * method (false).
	 */
	virtual bool shouldPlayout(GameState& state);

	/**
	 * Measures the "search value" of the given state, which is guaranteed to have already
	 * been visited at least once. This is the method which will determine the results of
	 * most calls to select() - in the normal case, select() will be choosing the child node
	 * which returns the maximal value of this function. This means that this method should
	 * return a higher value for states which are more desireable to continue a playout with,
	 * NOT a minimax value - that is, even if this state's parent is the minimizing player,
	 * this function should return a HIGHER value to represent the state in question being more
	 * valuable to choose to search next. This is DIFFERENT from estimateValue()'s operation -
	 * be careful to note the distinction when designing subclasses!
	 *
	 * This method's default implementation just returns an arbitrary constant value. In general,
	 * this is the function which causes different monte-carlo searches differ. For example, in
	 * UCT, it would simply return the results of the basic UCT formula applied to this state.
	 *
	 * It is given a default implementation only because some of the simpler searches, such as
	 * random monte-carlo, will not want to use the default select() method at all, favoring a
	 * simpler policy which does not require a maximizing operation. If select() does not need
	 * an evaluation function, there would be no need to require subclasses to write one.
	 *
	 * @param state The state to generate a search value for
	 * @return The "search value" of the given state, which should be higher if the state should
	 * be more likely to be considered for monte-carlo playout
	 */
	virtual double evaluate(GameState& state);

	/**
	 * Measures the "search value" of an unvisited child node of the given
	 * parent state. The details of why this is separate from evaluate() are primarily
	 * for performance reasons, but may also aid in logical separation (note that
	 * evaluate() is guaranteed not to be run on states that have never been previously
	 * visited).
	 *
	 * Like evaluate() (and not like estimateValue()), this method should always return values
	 * which will want to be maximized by either player. In general, the same constraints and
	 * recommendations for the evaluate() method apply to this method.
	 *
	 * The default implementation of this method returns DBL_MAX; this means that
	 * the default implementation of rollout-based monte-carlo search in general assumes
	 * that you will want to be searching every node at least once before considering any
	 * a second time. If this is not a property you want, it is highly advisable to
	 * override this method, whether to return a lower constant (as in the case of UCT-FPU)
	 * or some function of the parent node.
	 *
	 * @param parentState The parent of the blank state in question
	 * @return The search evaluation score for any unvisited child of that parentState
	 */
	virtual double evaluateFirst(GameState& parentState);

	/**
	 * Returns an empirical estimate of the reward value of the given state.
	 * This function should NOT take any random policy into account, like
	 * evaluate() is expected to - this function should represent purely the
	 * "exploitation" value of the state in question. This function is used
	 * to determine which state to identify as the current best available move,
	 * and may be used as one part of an evaluate() call.
	 *
	 * The returned value should be an estimate of minimax value of the state,
	 * which means that higher values should correspond to "better for the
	 * maximizing player (B)" and lower to being better for A. It is recommended
	 * that these values fall into the same bound as recommended for search() -
	 * by default, this is [0,1].
	 *
	 * The default implementation simply returns the empiricalValue property of
	 * the given state's searchData.
	 *
	 * @param state The state to estimate the value of
	 * @return An estimation of the state's minimax value
	 */
	virtual double estimateValue(GameState& state);

	/**
	 * Updates the internal searchData of the given state with the
	 * results of a single playout. This should be the only method which
	 * mutates states' searchData. This method will be called in the
	 * default implementations immediately before a search() over the state
	 * in question returns.
	 *
	 * The default implementation simply increments numVisits and updates
	 * empiricalValue such that it remains the running average of playoutValues
	 * encountered thus far.
	 * @param state The state to update
	 * @param playoutValue The search value to update the state with
	 */
	virtual void update(GameState& state, double playoutValue);

	/**
	 * Simulates the rest of the game in a random monte-carlo fashion. Nothing is
	 * stored in any of the generative child nodes, and in fact they might not even
	 * be generated at all if the tree allows for accurate guessing without generation.
	 *
	 * This method is closely coupled to the structure of the Gomba tree itself, and
	 * as such is not virtual so as to minimize the possible number of points which
	 * would need to change should the tree structure ever need to be modified.
	 *
	 * @param state The state in the tree to simulate play from
	 *
	 * @return The winning player from a random playout
	 */
	player_t simulate(GameState& state);


	/*
	 * See SearchAlgorithm::reset().
	 *
	 * Most subclass algorithms won't need to modify this.
	 */
	virtual void reset();
};

#endif /* ROLLOUTMONTECARLO_H_ */
