/*
 * UCT.h
 *
 *  Created on: 2010.03.24.
 *      Author: jnschaeffer
 *
 * Copyright 2010 Worcester Polytechnic Institute
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef UCT_H_
#define UCT_H_

#include "RolloutMonteCarlo.h"

/**
 * Represents an instance of the UCT algorithm. UCT recursively treats each internal node
 * of the tree as a separate UCT problem, selecting moves using the UCB1 policy. The reward
 * values from UCB1 for each node are the result of running UCT on internal nodes. UCT quickly
 * converges to a near-optimal move without excessive exploration of the tree, and will find
 * the optimal move eventually as it never stops exploring.
 */
class UCT: public RolloutMonteCarlo {
public:
	UCT(double simulationEffectiveness, double propogationConstant, double fpuConst = 10000);
	virtual ~UCT();

	virtual std::string toString() const;

protected:
	/**
	 * The current and previous best values from the algorithm.
	 */
	double curBest;

	/**
	 * The multiplier for the exploration half of the UCT equation. The original paper
	 * from Kocsis suggests sqrt(2) - empirical results from Mogo and Fuego suggest something
	 * near 0.7 or so for Go.
	 */
	double explorationConst;

	/**
	 * The bias value to be given to an unexplored node. Used by FPU to limit unnecessary
	 * exploration.
	 */
	double fpuConst;

	virtual double evaluate(GameState& root);
	virtual double evaluateFirst(GameState& parent);

	virtual double explorationBias(GameState& state);

};

#endif /* UCT_H_ */
