/*
 * UCTVInf.h
 *
 *  Created on: 8 avr. 2010
 *      Author: jnschaeffer
 *
 * Copyright 2010 Worcester Polytechnic Institute
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef UCTVINF_H_
#define UCTVINF_H_

#include "UCTV.h"

/**
 * UCT-V(inf) is a further extension to UCT-V, which assumes that the total number of moves
 * is greater than can be analyzed (effectively infinite). For this reason one of two selection
 * policies can be used:
 *
 * -Selection of K random arms which are used for the duration of the search.
 * -Selection of K random arms, followed by addition of random arms as a function of time
 *  (Arm-Increasing Rule)
 *
 * UCT-V is then applied to this subset of moves.
 */
class UCTVInf: public UCTV {
public:
	/**
	 * Generates a new instance of the UCT-V(inf) algorithm with the following parameters.
	 *
	 * @param horizon The UCT horizon to use.
	 * @param discount The discount factor of the MDP.
	 * @param effectiveness The effectiveness of the action simulation over a random move.
	 * @param k The number of arms to start using.
	 * @param beta A positive real number describing how "difficult" it will be to find an
	 * acceptably suboptimal move. Higher values will mean more exploration.
	 * @param useAIR True if the arm-increasing rule will be used.
	 */
	UCTVInf(double effectiveness,
			double propogation,
			double fpu,
			int k,
			double beta,
			bool useAIR);

	virtual ~UCTVInf();

	virtual std::string toString() const;

protected:
	/**
	 * Selects one arm from the K arms being used. If AIR is in place, this will also add
	 * arms to the set of arms currently being played if such an action is necessary.
	 *
	 * @param root The root of the tree to select from.
	 */
	action_t select(GameState& root);

	/**
	 * The number of arms to be played at any given time.
	 */
	unsigned int numArms;

	/**
	 * The beta parameter for UCT-V(inf).
	 */
	double beta;

	/**
	 * True if AIR is being used.
	 */
	bool useAIR;
};

#endif /* UCTVINF_H_ */
