/*
 * GameState.h
 *
 *  Created on: Mar 19, 2010
 *      Author: dbjorge
 *
 * Copyright 2010 Worcester Polytechnic Institute
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef GAMESTATE_H_
#define GAMESTATE_H_

#include <vector>
#include "Action.h"
#include "RandomGenerator.h"
#include "../search/SearchData.h"

class GameTreeGenerator;

/**
 * A particular state within the state-space tree of one game.
 *
 * This abstract class implements some of the backend functionality
 * not specific to actual tree generation, such as the caching of
 * child nodes.
 *
 * Concrete implementations will be required to have a means of generating
 * children on their own through the generateChild(size_t) method.
 * However, the generation method will not actually be called until
 * strictly necessary, and will be cached to avoid needing to generate
 * it again.
 *
 * Because of this limitation, but also because we'd like for the
 * tree to be internally consistent regardless of which nodes
 * are actually cached, care should be taken to ensure that a node's
 * children can be concretely recreated by it when necessary in a
 * manner which is fully deterministic.
 */
class GameState {
private:
	GameTreeGenerator & mGenerator;

	action_t mNumChildren;
	unsigned int mDepth;
	GameState* mParent;

	/** Array of size mNumChildren */
	GameState** mChildren;

	action_t mNextUnvisitedChild;
protected:
	/**
	 * Generates a brand new child for the given index. This
	 * method's output should be constant for the same object and
	 * index parameter!
	 *
	 * There is no need to worry about memory management of the
	 * pointer once created - all cached children will be deleted
	 * at destruction and clean time of the GameState superclass.
	 *
	 * @param index The index of the action/child to generate
	 * @return The newly generated child pointer.
	 */
	virtual GameState* generateChild(action_t index)=0;

	/**
	 * @return The GameTreeGenerator creating these states. Useful
	 * as a container of tree-wide info, like max depth or the RNG
	 */
	inline GameTreeGenerator & getTreeGenerator() const {
		return mGenerator;
	}
public:
	/**
	 * Constructor for root nodes. Automatically populates parent and
	 * depth as appropriate for a root node
	 */
	GameState(GameTreeGenerator& generator, action_t numChildren);

	/**
	 * Constructor for non-root nodes
	 */
	GameState(GameTreeGenerator& generator, GameState& parent, action_t numChildren);

	/**
	 * Destroys this node and all cached child nodes
	 */
	virtual ~GameState();

	/**
	 * @return 0 or 1, the player who would win if both were to play
	 * optimally from this state onwards. For consistency with the
	 * testing framework, this number should be completely accurate.
	 *
	 * For a leaf node in particular, this is guaranteed to be accurate.
	 *
	 * For some trees, this may be evaluated lazily, and it is thus not
	 * const by design.
	 */
	virtual player_t getWinner() const =0;

	/**
	 * @return Whether or not this node is a terminal state in the game
	 */
	inline bool isTerminal() const {
		return mNumChildren == 0;
	}

	/**
	 * @return The distance of the path from this node to the game's start state
	 */
	inline unsigned int getDepth() const {
		return mDepth;
	}

	/**
	 * @return The game state that immediately preceded this one, or NULL for
	 * the game's start state.
	 */
	inline GameState* getParent() const {
		return mParent;
	}

	/**
	 * @return The number of child nodes of this state
	 */
	inline action_t countChildren() const {
		return mNumChildren;
	}

	/**
	 * Retrieves the child at the given index, expanding and caching it if
	 * necessary. Try not to call this if you don't need to!
	 * @param index The index of the child (effectively represents the game action
	 * which will lead to the child state).
	 * @return The appropriate child game state
	 *
	 */
	GameState & getChild(action_t index);

	/**
	 * Returns the number of children which have already been visited.
	 *
	 * Note that this may only be an approximation - no guarantees are made
	 * about this result except its relationship to getVisitedChild(). However,
	 * since it will attempt to be accurate, it'll be faster to iterate over
	 * this and getVisitedChild() when the same value will be used to evaluate
	 * all non-visited children.
	 */
	action_t countVisitedChildren();

	/**
	 * Returns a child index which has not already been visited. Returns
	 * countChildren() if there are no visited children.
	 */
	action_t getNextUnvisitedAction();

	/**
	 * This may be faster than normal getChild by virtue of doing less
	 * error checking. However, it is required as a precondition that
	 * index be less than countVisitedChildren()
	 *
	 * Otherwise, same as getChild()
	 */
	GameState& getVisitedChild(action_t index);


	/**
	 * Returns a value-copy of the given child's SearchData structure, or a new blank
	 * one if the child hasn't previously been visited. If read-only access is sufficient,
	 * this can be much faster than using getChild().searchData for children which have
	 * not yet been expanded.
	 * @param index The action index of the child to get searchData for
	 * @return The SearchData of the given child, by value
	 */
	SearchData getChildSearchData(action_t index);

	/**
	 * Removes the child at the given index from the node cache and deletes it,
	 * if it existed
	 * @param index The child to un-cache
	 */
	void clean(action_t index);

	/**
	 * This is for whatever node-specific data any search algorithm would
	 * like to store. The methods of this class do not use this member, though
	 * it and any of its stored data will be destroyed whenever this node is.
	 */
	SearchData searchData;

	/**
	 * @return The player whose turn it is to play next
	 */
	inline player_t getPlayer() const {
		return (mDepth % 2 == 0) ? A : B;
	}

	/**
	 * @return Some measure of the difficulty of this node. This should be
	 * a number centered around .5, where higher than that means "Easier for
	 * player B" and lower means "Easier for player A"
	 */
	virtual double getDifficulty() const = 0;

	/**
	 * @return The most accurate measure of win probability of a random descendant
	 * leaf node available without actually generating a ton of child nodes.
	 */
	virtual double getWinProbability() const = 0;

	/**
	 * @return A "seed" value defining this node
	 */
	virtual seed_t getSeed() const = 0;

	/**
	 * Zeroes the child array and uses a new one from the stored
	 * tree generator. Useful for when the tree generator gets reset.
	 *
	 * Search algorithms shouldn't need to use this.
	 */
	void resetChildArray();
};

#endif /* GAMESTATE_H_ */
