/*
 * GameTree.h
 *
 *  Created on: Mar 19, 2010
 *      Author: dbjorge
 *
 * Copyright 2010 Worcester Polytechnic Institute
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef GAMETREE_H_
#define GAMETREE_H_

#include "GameState.h"
class GameTreeGenerator;

/**
 * GameTree is a wrapper around the root of an actual tree structure made of
 * some concrete GameState. It provides a means of holding onto the true root
 * of a full game even as the node representing the current state in play changes
 * with the progression of a game.
 */
class GameTree {

private:
	GameTreeGenerator& mGenerator;
	GameState* mRoot;
	GameState* mCurrent;
public:

	/**
	 * Constructs a new GameTree wrapper around the given starting node
	 * @param root The starting state of the new game.
	 */
	GameTree(GameTreeGenerator& generator, GameState* root);

	/**
	 * Destructs both this wrapper structure AND the wrapped tree node structure
	 */
	virtual ~GameTree();

	/**
	 * @return The true original root node of the entire game's tree.
	 */
	GameState & getRoot() const;

	/**
	 * @return The current state of the game. Guaranteed to be a decendant of
	 * the root.
	 */
	GameState & getCurrent() const;

	/**
	 * "Plays" the given index of the current state. Moved the current state of the
	 * game to the child of the current state referenced by the given index
	 * @param index The child index to move to
	 */
	void play(action_t index);

	/**
	 * Resets the current state of the game to the original root state and destroys
	 * the cache of nodes built up so far
	 */
	void reset();

	/**
	 * @return Whether the game is currently in a finished state
	 */
	bool isFinished() const;

	/**
	 * @return The number of nodes in the generated cache so far
	 */
	unsigned long cacheSize() const;
};

#endif /* GAMETREE_H_ */
