/*
 * GameTreeGenerator.cpp
 *
 *  Created on: Mar 22, 2010
 *      Author: dbjorge
 *
 * Copyright 2010 Worcester Polytechnic Institute
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "GameTreeGenerator.h"

GameTreeGenerator::GameTreeGenerator(
		action_t branchingFactor=100,
		unsigned int maxDepth=50,
		double difficultyBias=.5,
		seed_t seed=(seed_t)time(0),
		unsigned int blockSize=1000) :
	mBranchingFactor(branchingFactor),
	mMaxDepth(maxDepth),
	mDifficultyBias(difficultyBias),
	mGenerator(seed),
	mStatePool(sizeof(GameStateType), blockSize, false),
	mChildArrayPool(sizeof(GameStateType*)*mBranchingFactor, blockSize, true)
{ }

GameTreeGenerator::~GameTreeGenerator()
{ }

unsigned int GameTreeGenerator::getMaxDepth() const {
	return mMaxDepth;
}

GameTree* GameTreeGenerator::generate() {
	GameState* root = new MinimaxGameState(
			*this,
			mBranchingFactor,
			mGenerator.nextSeed(),
			A, //A plays first - if B were to win, that means A doesn't have an optimal move, which screws up statistics
			mDifficultyBias);
	return new GameTree(*this, root);
}

RandomGenerator & GameTreeGenerator::getRNG() {
	return mGenerator;
}

/**
 * Returns a pointer to a block of memory which can be used to hold the next
 * game state
 */
void* GameTreeGenerator::nextState() {
	return mStatePool.next();
}

/**
 * Returns a pointer to a block of memory big enough for a GameState*[branchingFactor]
 */
void* GameTreeGenerator::nextChildArray() {
	return mChildArrayPool.next();
}

/**
 * Resets the memory pool to allow reuse
 */
void GameTreeGenerator::reset() {
	mStatePool.freeAll();
	mChildArrayPool.freeAll();
}


/**
 * @return The number of nodes in the generated cache so far
 */
unsigned long GameTreeGenerator::cacheSize() const {
	return mStatePool.size();
}

