/*
 * MemoryPool.cpp
 *
 *  Created on: Mar 26, 2010
 *      Author: dbjorge
 *
 * Copyright 2010 Worcester Polytechnic Institute
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "MemoryPool.h"

//memset
#include <string.h>


MemoryPool::MemoryPool(size_t objectSize, unsigned int blockSize, bool zeroOnFree) :
	mObjectSize(objectSize),
	mBlockSize(blockSize),
	mZeroOnFree(zeroOnFree),
	mBlocks(),
	mNumBlocks(0),
	mNextStateIndex(0),
	mNextBlockIndex(0),
	mCurrentBlock(NULL),
	mBlocksIterator(mBlocks.begin())
{ }

MemoryPool::~MemoryPool() {
	for(mBlocksIterator = mBlocks.begin(); mBlocksIterator != mBlocks.end(); mBlocksIterator++) {
		free(*mBlocksIterator);
	}
}


void MemoryPool::freeAll() {
	mNextStateIndex = 0;
	mNextBlockIndex = 0;
	mCurrentBlock = NULL;
	mBlocksIterator = mBlocks.begin();

	if(mZeroOnFree) {
		for(std::list<void*>::iterator it = mBlocks.begin(); it != mBlocks.end() ; it++) {
			memset(*it, 0, mObjectSize * mBlockSize);
		}
	}
}

void* MemoryPool::next() {
	if(mNextStateIndex >= mBlockSize) mNextStateIndex = 0;

	if(mNextStateIndex == 0) {
		if(mNextBlockIndex >= mNumBlocks) {
			if(mZeroOnFree) {
				mCurrentBlock = calloc(mBlockSize, mObjectSize);
			} else {
				mCurrentBlock = malloc(mBlockSize * mObjectSize);
			}

			if(mCurrentBlock == NULL) {
				throw new std::bad_alloc();
			}
			mNumBlocks++;
			mBlocks.push_back(mCurrentBlock);
		} else {
			mCurrentBlock = *mBlocksIterator;
			mBlocksIterator++;
		}
		mNextBlockIndex++;
	}

	//The char* cast means pointer arithmetic deals in single bytes
	//Since we explicitly know mObjectSize, that's what we want.
	void* next = (void*) (((char*) mCurrentBlock) + (mNextStateIndex * mObjectSize));
	mNextStateIndex++;

	return next;
}

unsigned long MemoryPool::size() const {
	if(mNextBlockIndex == 0) return 0;
	return ((unsigned long) (mNextBlockIndex - 1)) * mBlockSize + mNextStateIndex;
}
