/*
 * MinimaxGameState.h
 *
 *  Created on: Mar 22, 2010
 *      Author: dbjorge
 *
 * Copyright 2010 Worcester Polytechnic Institute
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef MINIMAXGAMESTATE_H_
#define MINIMAXGAMESTATE_H_

#include "GameState.h"
#include "RandomGenerator.h"

class GameTreeGenerator;

/**
 * A particular GameState type which is capable of pre-determining
 * the minimax state of the whole tree without having to generate the
 * whole tree at runtime. This is the "default" GameState implementation
 * for the early versions of the Gomba library.
 */
class MinimaxGameState: public GameState {
private:
	/**
	 * The original seed for this game state. Not really needed, but useful as an identifier
	 */
	seed_t mSeed;

	/**
	 * The PRNG for this state's needs. This needs to be small! A LCG or a LFSR would
	 * be okay, something with minimal internal state.
	 */
	RandomGenerator mRng;
	/**
	 * A measure of how difficult this node is for each player to win. The closer to
	 * 0, the easier it will be for player 0 to win, and vice versa. This is used to
	 * determine the distribution of which children will win for which players, when
	 * random choice is involved in that decision.
	 */
	double mDifficulty;

	/**
	 * The predetermined minimax winner from this state, were both players to play
	 * out the rest of the game tree optimally
	 */
	player_t mWinner;

	/**
	 * The seed value for the first child of this node. This is generated randomly
	 * at construction time based on our own seed - subsequent children are simply
	 * this value plus the child's action.
	 */
	seed_t mChildSeed;

	/**
	 * If this node will require at least one child to be of a particular Winner value
	 * for the sake of the minimax tree construction, this value is the action of the
	 * random child that will be forced to that value.
	 *
	 * This value is undefined (but probably zero) if we instead need to force all
	 * children to one Winner value.
	 */
	action_t mForcedChild;

	/**
	 * If this node will require all children to be of a particular Winner value,
	 * it will be this value (one of A or B). If this value is NEITHER, the choice
	 * will not be forced (except for mForcedChild)
	 */
	player_t mForcedWinner;

	/**
	 * Private initialization routines for constructors to pass to
	 */
	void init();

public:
	/**
	 * General constructor for Minimax Game States.
	 * @param tree The tree this is a part of. Provides access to attributes, RNG
	 * @param numChildren The number of actions/children to be available from this node
	 * @param seed The seed which should be used to generate this node's attributes and children
	 * @param winner The winning player from this node should both play optimally. If NEITHER,
	 * the actual winner will be determined randomly based on the generated difficulty
	 * @param parent The parent state
	 * @param parentDifficulty The difficulty of the parent state (used as a baseline for ours)
	 */
	MinimaxGameState(
			GameTreeGenerator& tree,
			GameState& parent,
			action_t numChildren,
			seed_t seed,
			player_t winner,
			double parentDifficulty);

	/**
	 * Root constructor for Minimax Game States. As normal, but there is no
	 * parent state
	 */
	MinimaxGameState(
			GameTreeGenerator& tree,
			action_t numChildren,
			seed_t seed,
			player_t winner,
			double initialDifficulty);

	MinimaxGameState* generateChild(action_t index);
	player_t getWinner() const;
	double getDifficulty() const;
	double getWinProbability() const;
	seed_t getSeed() const;
};

#endif /* MINIMAXGAMESTATE_H_ */
