/*
 * RandomGenerator.h
 *
 *  Created on: Mar 23, 2010
 *      Author: dbjorge
 *
 * Copyright 2010 Worcester Polytechnic Institute
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef RANDOMGENERATOR_H_
#define RANDOMGENERATOR_H_

#include <map>
#include <boost/random/lagged_fibonacci.hpp>
#include <boost/random.hpp>
#include "Action.h"

typedef uint32_t seed_t;

/**
 * Wrapper around a real random number generator to provide intermediary
 * caching and thread safety, as well as convenience methods.
 *
 * The PRNG used by this class is NOT SUITABLE for anything that requires
 * more than a cursory approximation of randomness. It is used in tree generation
 * because the perculiarities of tree generation require not only that number
 * generation be extremely fast but that seeding be extremely fast. Thus,
 * this generator uses a simple Linear Congruential Generator for an implementation.
 *
 * LCGs have MANY known statistical flaws. Again, you should NOT be using this
 * class in anything that doesn't strictly require both very fast generation AND
 * very fast seeding. In particular, this generator is NOT SUITABLE for Monte-Carlo
 * simulations because its results exhibit serial correlation.
 */
class RandomGenerator {
private:
	seed_t mState;

	double nextNormal();
	template <typename t>
	t next();
public:
	/**
	 * Creates a new random number generator with a starting seed based
	 * on the current hardware time.
	 */
	RandomGenerator();

	/**
	 * Creates a new random number generator based on the given starting seed
	 */
	RandomGenerator(seed_t startingSeed);

	~RandomGenerator();

	/**
	 * Randomly creates a new number which is a slight variation
	 * on the original number.
	 * @param original The original starting value to vary from. [0..1]
	 * @return The newly varied number, also in [0..1]
	 */
	double nextVaried(double original);

	/**
	 * @return A uniformly random double in the range [0..1]
	 */
	double nextReal();

	/**
	 * Returns the nth seed starting from the given seed. This method neither
	 * USES nor MODIFIES the state of this generator. That is, consecutive calls
	 * to this method will return the same thing for the same argument.
	 */
	seed_t nthSeed(seed_t startSeed, action_t n);

	/**
	 * Chooses a random action in [0..numActions]. This function
	 * is able to cache some data structures for each different numActions
	 * value it is called with, which means ideally numActions will stay fairly
	 * constant.
	 * @param numActions The number of available actions, in [0..]
	 * @return A randomly chosen action, in [0..numActions]
	 */
	action_t nextAction(action_t numActions);

	/**
	 * Generates a new seed value which this class can be initialized to.
	 * Generally this will be used to preserve state with a small single
	 * value.
	 * @return A new seed value, valid for this generator.
	 */
	seed_t nextSeed();

	/**
	 * Generates a player based on a difficulty. Uniformly weighted - that is,
	 * the probability that player B will be chosen is exactly the difficulty
	 * value.
	 * @param difficulty The likelyhood of chosing player B over A
	 * @return The chosen random player
	 */
	player_t nextPlayer(double difficulty);

	/**
	 * Seeds the generator with the given value. Note that if you need it
	 * to stay on this seed to generate a few values, you should lock it for
	 * the session with startSession(seed), unless you actually know that
	 * you aren't multithreading.
	 * @param seed The seed value to seed with
	 */
	void seed(seed_t seed);

	unsigned short nextBit();
};

#endif /* RANDOMGENERATOR_H_ */
