/*
 * AlgorithmRegistration.h
 *
 *  Created on: Apr 20, 2010
 *      Author: dbjorge
 *
 * Copyright 2010 Worcester Polytechnic Institute
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef ALGORITHMREGISTRATION_H_
#define ALGORITHMREGISTRATION_H_

/**
 * Defines a bunch of macros and a global function or two to deal with allowing simple
 * registration of algorithms with names. Individual algorithm registrations
 * belong in algorithms/AllAlgorithms.h, which #includes this file
 */

#include <map>
#include <vector>
#include <list>
#include "SearchAlgorithm.h"

namespace algreg {

typedef SearchAlgorithm* (*algGenerator_t)(std::vector<double>&);

class AlgorithmRegistrar {
public:
	void registerAlgs();
};

AlgorithmRegistrar registrar;
std::map<std::string, std::list<algGenerator_t> > registeredAlgMap;

bool registrationComplete = false;

void registerAlg(std::string name, algGenerator_t generator) {
	if(registeredAlgMap.count(name) == 0) {
		registeredAlgMap[name] = std::list<algGenerator_t>();
	}

	registeredAlgMap[name].push_back(generator);
};

SearchAlgorithm* generateRegisteredAlgorithm(std::string name, std::vector<double>& args) {
	registrar.registerAlgs();

	if(registeredAlgMap.count(name) == 0) return NULL;

	SearchAlgorithm* potentialAlg = NULL;

	std::list<algGenerator_t>& generators = registeredAlgMap[name];
	for(std::list<algGenerator_t>::iterator it = generators.begin() ; it != generators.end() ; it++) {
		potentialAlg = (*(*it))(args);
		if(potentialAlg != NULL) return potentialAlg;
	}

	return NULL;
};

#define REGISTER_ALG_N(name, clazz, n, ...) \
	class name##_##clazz##_##n##_generator { \
	public: \
	static SearchAlgorithm* generate(std::vector<double>& args) { \
		if(args.size() < n) return NULL; \
		return new clazz(__VA_ARGS__); \
	}}; \
	algreg::registerAlg(#name, &(name##_##clazz##_##n##_generator::generate))

#define REGISTER_ALG_0(name, clazz) \
	class name##_##clazz##_##0##_generator { \
	public: \
	static SearchAlgorithm* generate(std::vector<double>& args) { \
		return new clazz(); \
	}}; \
	algreg::registerAlg(#name, &(name##_##clazz##_##0##_generator::generate))

#define REGISTER_ALG_1(name, clazz) REGISTER_ALG_N(name, clazz, 1, args[0])
#define REGISTER_ALG_2(name, clazz) REGISTER_ALG_N(name, clazz, 2, args[0], args[1])
#define REGISTER_ALG_3(name, clazz) REGISTER_ALG_N(name, clazz, 3, args[0], args[1], args[2])
#define REGISTER_ALG_4(name, clazz) REGISTER_ALG_N(name, clazz, 4, args[0], args[1], args[2], args[3])
#define REGISTER_ALG_5(name, clazz) REGISTER_ALG_N(name, clazz, 5, args[0], args[1], args[2], args[3], args[4])

#define BEGIN_REGISTER_ALGS \
	void algreg::AlgorithmRegistrar::registerAlgs() { \
		if(algreg::registrationComplete) return;
#define END_REGISTER_ALGS \
		; algreg::registrationComplete = true; \
	}

} // namespace algreg

#endif /* ALGORITHMREGISTRATION_H_ */
